# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import glob
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko 


class ModisChecks(Checks):
    """
    Checks specific to Modis
    """
    def __init__(self, indir, file):
        super().__init__(indir, file)
        self.dataset = "modis"

    def check_that_file_exists_modis(self):
        """
        MOdis check because we don't know the exact name of the file
        """
        hdffiles = glob.glob("{}/*.hdf".format(self.indir))
        print(self.indir)
        print(hdffiles)

        if len(hdffiles) == 1:
            self.filepath = hdffiles[0]
        else:
            return "TEST: File does not exist in the destination {}".format(self.indir)


    def check_variable_names(self):
        print("Checking variable names")
        ds = xr.open_dataset(self.filepath)
        variable_name = self.file.split("_")[0]
        
        try: 
            ds["longitude"]
        except KeyError:
            return "Longitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds["latitude"]
        except KeyError:
            return "Latitude variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds["time"]
        except KeyError:
            return "Time variable doesn't exist in file - {}".format(self.filepath)
        
        try: 
            ds[variable_name]
        except KeyError:
            return "{} variable doesn't exist in file - {}".format(variable_name, self.filepath)
        
        
        if ds[variable_name].min() < 0:
            return "File has values less than zero - {}".format(self.filepath)
        
        if ds[variable_name].max() > 100:
            return "File has maximum outliers - {}".format(self.filepath)
        

        

    
    def modis_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists_modis()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)
    
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")


    def modis_formatted_files_check(self):
        """
        Checking formatted Modis files
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size()
                if file_size != None:
                    error_list.append(file_size)
                    self.send_email_over_ssh(error_list)

                file_variables = self.check_variable_names()
                if file_variables != None:
                    error_list.append(file_variables)
                    self.send_email_over_ssh(error_list)
    
                if error_list != []:
                    print(error_list)
                else:
                    print("No errors reported")





if __name__ == "__main__":
    print(sys.argv)
    datatype = sys.argv[1]

    #options = getOptions(sys.argv[1:])
    today = datetime.today()
    yesterday = (today - timedelta(days=1)).strftime("%Y%m%d")

    base_indir = "/esarchive/obs/nasa/{}-col61/original_files".format(datatype)

    for i in range(2, 10):
        date = (today - timedelta(days=i)).strftime("%Y%m%d")
    
        #download data check:
        #daily
        
        
        date_new = datetime.strptime(date, "%Y%m%d")
        juliandate = date_new.strftime("%Y%j")
        indir = "{}/{}_D3/{}/{}/".format(base_indir, datatype.upper(), juliandate[:4], juliandate[4:])
        mod_dwl = ModisChecks(indir, file="")
        mod_dwl.modis_original_files_check()
        
    
        # daily
        # Formatted file check
        path_dir = "/esarchive/obs/nasa/{}-col61/daily_mean".format(datatype)
        element = "od550aero"
        indir = path_dir + "/{}".format(element)
        file = "{}_{}.nc".format(element, date)
        print("Checking file {}".format(file))
        mod_fmt = ModisChecks(indir, file)
        mod_fmt.modis_formatted_files_check()

    juliandate = (today - timedelta(days=5)).replace(day=1).strftime("%Y%j")
    print(juliandate)
    indir = "{}/{}_M3/{}/{}/".format(base_indir, datatype.upper(), juliandate[:4], juliandate[4:])
    mod_dwl = ModisChecks(indir, file="")
    mod_dwl.modis_original_files_check()
    
    
    # monthly
    # Formatted file check
    
    path = "/esarchive/obs/nasa/{}-col61/monthly_mean".format(datatype)
    element = "od550aero"
    date = (today - timedelta(days=5)).strftime("%Y%m%d")
    indir = path + "/{}".format(element)
    file = "{}_{}01.nc".format(element, date[:6])
    print("Checking file {}".format(file))
    mod_fmt = ModisChecks(indir, file)
    mod_fmt.modis_formatted_files_check()

    
    sys.exit()